@@ -10,7 +10,6 @@
 import os
 import sys
 import json
-import ConfigParser
 from vdirsyncer.sync import sync
 from vdirsyncer.utils import expand_path, split_dict, parse_options
 from vdirsyncer.storage import storage_names
@@ -18,11 +17,17 @@ import vdirsyncer.log as log
 import argvard
 
 
-cli_logger = log.get('cli')
+try:
+    from ConfigParser import RawConfigParser
+except ImportError:
+    from configparser import RawConfigParser
+
+
+cli_logger = log.get(__name__)
 
 
 def load_config(fname, pair_options=('collections', 'conflict_resolution')):
-    c = ConfigParser.RawConfigParser()
+    c = RawConfigParser()
     c.read(fname)
 
     get_options = lambda s: dict(parse_options(c.items(s)))
@@ -69,30 +74,59 @@ def save_status(basepath, pair_name, status):
             f.write('\n')
 
 
-def storage_instance_from_config(config):
+def storage_instance_from_config(config, description=None):
+    '''
+    :param config: A configuration dictionary to pass as kwargs to the class
+        corresponding to config['type']
+    :param description: A name for the storage for debugging purposes
+    '''
+
     config = dict(config)
     storage_name = config.pop('type')
     cls = storage_names[storage_name]
     try:
         return cls(**config)
-    except TypeError as e:
-        import inspect
-        x = cli_logger.critical
-        spec = inspect.getargspec(cls.__init__)
-        required_args = set(spec.args[:-len(spec.defaults)])
+    except TypeError:
+        all, required = get_init_args(cls)
+        given = set(config)
+        missing = required - given
+        invalid = given - all
+
+        cli_logger.critical('error: Failed to initialize {}'
+                            .format(description or storage_name))
+
+        if not missing and not invalid:
+            cli_logger.exception('')
+
+        if missing:
+            cli_logger.critical(
+                u'error: {} storage requires the parameters: {}'
+                .format(storage_name, u', '.join(missing)))
+
+        if invalid:
+            cli_logger.critical(
+                u'error: {} storage doesn\'t take the parameters: {}'
+                .format(storage_name, u', '.join(invalid)))
 
-        x(str(e))
-        x('')
-        x('Unable to initialize storage {}.'.format(storage_name))
-        x('Here are the required arguments for the storage:')
-        x(list(required_args - {'self'}))
-        x('Here are the optional arguments:')
-        x(list(set(spec.args) - required_args))
-        x('And here are the ones you gave: ')
-        x(list(config))
         sys.exit(1)
 
 
+def get_init_args(cls):
+    from vdirsyncer.storage.base import Storage
+    import inspect
+
+    if cls is Storage:
+        return set(), set()
+
+    spec = inspect.getargspec(cls.__init__)
+    all = set(spec.args[1:])
+    required = set(spec.args[1:-len(spec.defaults)])
+    supercls = next(x for x in cls.__mro__[1:] if hasattr(x, '__init__'))
+    s_all, s_required = get_init_args(supercls)
+
+    return all | s_all, required | s_required
+
+
 def main():
     env = os.environ
 
@@ -150,6 +184,7 @@ def _main(env, file_cfg):
         normal user.
 
         '''
+        verbosity = verbosity.upper()
         x = getattr(log.logging, verbosity, None)
         if x is None:
             raise ValueError(u'Invalid verbosity value: {}'.format(verbosity))
@@ -184,23 +219,46 @@ def _main(env, file_cfg):
             config_a.update(all_storages[a_name])
             config_b = dict(storage_defaults)
             config_b.update(all_storages[b_name])
-            a = storage_instance_from_config(config_a)
-            b = storage_instance_from_config(config_b)
 
-            def x(a=a, b=b, pair_name=pair_name, collection=collection):
-                status_name = \
-                    '_'.join(filter(bool, (pair_name, collection)))
-                pair_description = \
-                    ' from '.join(filter(bool, (collection, pair_name)))
-                cli_logger.info('Syncing {}'.format(pair_description))
-                status = load_status(general['status_path'], status_name)
-                sync(a, b, status,
-                     pair_options.get('conflict_resolution', None))
-                save_status(general['status_path'], status_name, status)
-            actions.append(x)
+            actions.append({
+                'config_a': config_a,
+                'config_b': config_b,
+                'pair_name': pair_name,
+                'collection': collection,
+                'pair_options': pair_options,
+                'general': general
+            })
 
-        for action in actions:
-            action()
+        processes = general.get('processes', 0) or len(actions)
+        cli_logger.debug('Using {} processes.'.format(processes))
+
+        if processes == 1:
+            cli_logger.debug('Not using multiprocessing.')
+            map(_sync_collection, actions)
+        else:
+            cli_logger.debug('Using multiprocessing.')
+            from multiprocessing import Pool
+            p = Pool(processes=general.get('processes', 0) or len(actions))
+            p.map_async(_sync_collection, actions).get(10**9)
 
     app.register_command('sync', sync_command)
     app()
+
+
+def _sync_collection(x):
+    return sync_collection(**x)
+
+
+def sync_collection(config_a, config_b, pair_name, collection, pair_options,
+                    general):
+    status_name = '_'.join(filter(bool, (pair_name, collection)))
+    pair_description = ' from '.join(filter(bool, (collection, pair_name)))
+
+    a = storage_instance_from_config(config_a, pair_description)
+    b = storage_instance_from_config(config_b, pair_description)
+
+    cli_logger.info('Syncing {}'.format(pair_description))
+    status = load_status(general['status_path'], status_name)
+    sync(a, b, status,
+         pair_options.get('conflict_resolution', None))
+    save_status(general['status_path'], status_name, status)
