@@ -11,14 +11,13 @@ from .base import Storage, Item
 from .http import prepare_auth, prepare_verify, USERAGENT
 from .. import exceptions
 from .. import log
-from ..utils import request, get_password
+from ..utils import request, get_password, urlparse
 import requests
 import datetime
-import urlparse
 from lxml import etree
 
 
-dav_logger = log.get('storage.dav')
+dav_logger = log.get(__name__)
 
 CALDAV_DT_FORMAT = '%Y%m%dT%H%M%SZ'
 CONFIG_DT_FORMAT = '%Y-%m-%d'
@@ -166,7 +165,7 @@ class DavStorage(Storage):
         hrefs_left = set(hrefs)
         for element in root.iter('{DAV:}response'):
             href = self._normalize_href(
-                element.find('{DAV:}href').text.decode(response.encoding))
+                element.find('{DAV:}href').text)
             raw = element \
                 .find('{DAV:}propstat') \
                 .find('{DAV:}prop') \
@@ -335,7 +334,7 @@ class CaldavStorage(DavStorage):
             end = end.strftime(CALDAV_DT_FORMAT)
 
             timefilter = ('<C:time-range start="{start}" end="{end}"/>'
-                .format(start=start, end=end))
+                          .format(start=start, end=end))
         else:
             timefilter = ''
 
