@@ -19,7 +19,8 @@ import itertools
 
 import vdirsyncer.exceptions as exceptions
 import vdirsyncer.log
-sync_logger = vdirsyncer.log.get('sync')
+from .utils import iteritems, itervalues
+sync_logger = vdirsyncer.log.get(__name__)
 
 
 def prepare_list(storage, href_to_status):
@@ -58,18 +59,18 @@ def sync(storage_a, storage_b, status, conflict_resolution=None):
     '''
     a_href_to_status = dict(
         (href_a, (uid, etag_a))
-        for uid, (href_a, etag_a, href_b, etag_b) in status.iteritems()
+        for uid, (href_a, etag_a, href_b, etag_b) in iteritems(status)
     )
     b_href_to_status = dict(
         (href_b, (uid, etag_b))
-        for uid, (href_a, etag_a, href_b, etag_b) in status.iteritems()
+        for uid, (href_a, etag_a, href_b, etag_b) in iteritems(status)
     )
     # href => {'etag': etag, 'item': optional item, 'uid': uid}
     list_a = dict(prepare_list(storage_a, a_href_to_status))
     list_b = dict(prepare_list(storage_b, b_href_to_status))
 
-    a_uid_to_href = dict((x['uid'], href) for href, x in list_a.iteritems())
-    b_uid_to_href = dict((x['uid'], href) for href, x in list_b.iteritems())
+    a_uid_to_href = dict((x['uid'], href) for href, x in iteritems(list_a))
+    b_uid_to_href = dict((x['uid'], href) for href, x in iteritems(list_b))
     del a_href_to_status, b_href_to_status
 
     storages = {
@@ -77,7 +78,7 @@ def sync(storage_a, storage_b, status, conflict_resolution=None):
         'b': (storage_b, list_b, b_uid_to_href)
     }
 
-    actions = get_actions(storages, status)
+    actions = list(get_actions(storages, status))
 
     for action in actions:
         action(storages, status, conflict_resolution)
@@ -172,13 +173,11 @@ def action_conflict_resolve(uid):
 
 
 def get_actions(storages, status):
-    actions = []
-
     storage_a, list_a, a_uid_to_href = storages['a']
     storage_b, list_b, b_uid_to_href = storages['b']
 
-    uids_a = (x['uid'] for x in list_a.itervalues())
-    uids_b = (x['uid'] for x in list_b.itervalues())
+    uids_a = (x['uid'] for x in itervalues(list_a))
+    uids_b = (x['uid'] for x in itervalues(list_b))
     handled = set()
     for uid in itertools.chain(uids_a, uids_b, status):
         if uid in handled:
@@ -191,25 +190,23 @@ def get_actions(storages, status):
         b = list_b.get(href_b, None)
         if uid not in status:
             if a and b:  # missing status
-                actions.append(action_conflict_resolve(uid))
+                yield action_conflict_resolve(uid)
             elif a and not b:  # new item was created in a
-                actions.append(action_upload(uid, 'a', 'b'))
+                yield action_upload(uid, 'a', 'b')
             elif not a and b:  # new item was created in b
-                actions.append(action_upload(uid, 'b', 'a'))
+                yield action_upload(uid, 'b', 'a')
         else:
             _, status_etag_a, _, status_etag_b = status[uid]
             if a and b:
                 if a['etag'] != status_etag_a and b['etag'] != status_etag_b:
-                    actions.append(action_conflict_resolve(uid))
+                    yield action_conflict_resolve(uid)
                 elif a['etag'] != status_etag_a:  # item was updated in a
-                    actions.append(action_update(uid, 'a', 'b'))
+                    yield action_update(uid, 'a', 'b')
                 elif b['etag'] != status_etag_b:  # item was updated in b
-                    actions.append(action_update(uid, 'b', 'a'))
+                    yield action_update(uid, 'b', 'a')
             elif a and not b:  # was deleted from b
-                actions.append(action_delete(uid, 'a'))
+                yield action_delete(uid, 'a')
             elif not a and b:  # was deleted from a
-                actions.append(action_delete(uid, 'b'))
+                yield action_delete(uid, 'b')
             elif not a and not b:  # was deleted from a and b
-                actions.append(action_delete(uid, None))
-
-    return actions
+                yield action_delete(uid, None)
